package main

import (
  "fmt"
  "os"
  "strconv"
  "strings"
)

type data struct {
  g, m, a int
}

/* Data una stringa della forma gg/mm/aaaa restituisce la data corrispondente */
func strToData(s string) (data, bool) {
    p1 := strings.IndexRune(s, '/')
    if p1 < 0 {
        return data{}, false
    }
    p2 := strings.IndexRune(s[p1 + 1:], '/')
    if p2 < 0 {
        return data{}, false
    }
    p2 += p1 + 1
    g, err := strconv.Atoi(s[:p1])
    if err != nil {
        return data{}, false
    }
    m, err := strconv.Atoi(s[p1 + 1:p2])
    if err != nil {
        return data{}, false
    }
    a, err := strconv.Atoi(s[p2 + 1:])
    if err != nil {
        return data{}, false
    }
    // I should check that g/m/a is a correct date
    return data{g, m, a}, true
}

/* Restituisce true se e solo se la data d1 precede strettamente la data d2 */
func vienePrimaDi(d1, d2 data) bool {
  if d1.a != d2.a {
    return d1.a < d2.a
  }
  if d1.m != d2.m {
    return d1.m < d2.m
  }
  return d1.g < d2.g
}

/* Restituisce la prima data in ordine cronologico fra le date della slice d,
  insieme al primo indice in cui questa data compare.
  Panic se la slice è vuota. */
func prima(d []data) (data,int) {
  candidato := d[0]   // Alterantivamente, inizializza a +infinito e fai partire il ciclo da 0
  indiceCandidato := 0
  for i := 1; i < len(d); i++ {
    if vienePrimaDi(d[i], candidato) {
      candidato = d[i]
      indiceCandidato = i
    }
  }
  return candidato, indiceCandidato
}

func main() {
  var date []data
  for i := 1; i < len(os.Args); i++ {
    d, ok := strToData(os.Args[i])
    if ok {
      date = append(date, d)
    } else {
      fmt.Println("Errore: non sono stato in grado di fare parsing della data", os.Args[i])
      fmt.Println("---ignoro questa data")
    }
  }
  d, i := prima(date)
  fmt.Println("La data minima è ", d)
  fmt.Println("e la sua ultima occorrenza è ", i)

}
